;**********************************************************************************
;Manchester Encoder for wireless conceptXM1 link
;**********************************************************************************
;type: subroutine <call/ret>
;-> byte to encode:  zl
;-> loop cnt: zh
;-> encoded (8bit): yl
;cycles: .....
;**********************************************************************************

	;*************************************
	;start conditions - call entry
	;*************************************
_encodeMAN:
	clr	yl		;16bit encoded LOW
	ldi	zh,4	;8	;loop cnt


	;*************************************
	;encoder loop
	;*************************************
_encodeMANloop:
	;loop fertig ?
	tst	zh
	breq	_encodeMANloopBK
	dec	zh
	rol	zl			;rotate byte left (carry to bit 0 / bit7 to carry)
	brcc _HLedge	;bit7 =0
	rjmp _LHedge	;bit7 =1

	;*************************************
	;encoder loop ready - return f. call
	;*************************************
_encodeMANloopBK:
	ret




	;*********************************************************
	;carry im quellbyte =0  <=> High/Low-Edge im encoded byte 
	;*********************************************************
_HLedge:
	sec		
	;LSB nach links
	rol	yl
	;jetzt clock LOW
	clc
	;LSB nach links
	rol	yl
	rjmp	_encodeMANloop


	;*********************************************************
	;carry im quellbyte =1  <=> Low/High-Edge im encoded byte 
	;*********************************************************
_LHedge:
	;clock LOW
	clc		
	;LSB nach links
	rol	yl
	;jetzt clock HIGH
	sec		
	;LSB nach links
	rol	yl
	rjmp	_encodeMANloop














;**********************************************************************************
;Manchester Decoder for wireless MultiNET0.x link
;**********************************************************************************
;type: subroutine <call/ret>
;-> byte to decode:  zl
;-> loop cnt: zh
;-> coded (8bit): yl
;cycles: .....
;**********************************************************************************


	;*************************************
	;start conditions - call entry
	;*************************************
_decodeMAN:
	clr		zl		;8bit decoded 
	ldi		zh,4	;loop cnt



	;*************************************
	;decoder loop
	;*************************************
_decodeMANloop:
	;loop fertig ?
	tst		zh
	breq	_decodeMANloopBK
	dec		zh
	;start test edge?
;shift LSB right+carry
	lsr		yl
	brcs	_LHedgeD		;Low.....L/H edge?
	rjmp	_HLedgeD		;High....H/L edge?



	;****************************************************************
	;H/L edge?
	;****************************************************************
	;MSB right
_HLedgeD:
	;LSB right
	lsr		yl			;shift LSB right 
	brcs	_DECzero	;H/L edge erkannt !	
	;eigentlich ist das
	;ein fehlerhafter 
	;zustand !!!
	rjmp	_DECone

	;decoder byte aktualisieren
	;H/L edge == 0
_DECzero:
	;shift decoded byte right
	lsr	zl
	cbr	zl,1<<7

	rjmp	_decodeMANloop	
	




	;****************************************************************
	;L/H edge?
	;****************************************************************
	;MSB right
_LHedgeD:
	;LSB right
	lsr		yl			;shift LSB right 
	brcc	_DECone		;L/H edge erkannt !	
	;eigentlich ist das
	;ein fehlerhafter 
	;zustand !!!
	rjmp	_DECzero	
	;decoder byte aktualisieren
	;L/H edge == 1
_DECone:
	;shift decoded byte right
	lsr	zl
	sbr	zl,1<<7

	rjmp	_decodeMANloop	
	
		

	;*************************************
	;decoder loop ready - return f. call
	;*************************************
_decodeMANloopBK:
	ret





